from __future__ import annotations

from typing import Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.checks.enums import BlockType
from checkov.openapi.checks.base_openapi_check import BaseOpenapiCheck


class Oauth2SecurityRequirement(BaseOpenapiCheck):
    def __init__(self) -> None:
        id = "CKV_OPENAPI_2"
        name = "Ensure that if the security scheme is not of type 'oauth2', the array value must be empty"
        categories = [CheckCategories.API_SECURITY]
        supported_resources = ['security']
        self.irrelevant_keys = ['__startline__', '__endline__']
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_resources,
                         block_type=BlockType.DOCUMENT)

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:  # type:ignore[override] # return type is different than the base class
        security_values = conf.get("security", [{}])
        security_definitions = conf.get("securityDefinitions", {})
        non_oauth2_keys = []

        for auth_key, auth_dict in security_definitions.items():
            if auth_key in self.irrelevant_keys:
                continue
            auth_type = auth_dict.get("type")
            if auth_type.lower() != "oauth2":
                non_oauth2_keys.append(auth_key)

        for auth_dict in security_values:
            if not isinstance(auth_dict, dict):
                return CheckResult.UNKNOWN, conf
            for key, auth_list in auth_dict.items():
                if key in self.irrelevant_keys:
                    continue
                if key in non_oauth2_keys and auth_list:
                    return CheckResult.FAILED, security_values

        return CheckResult.PASSED, conf


check = Oauth2SecurityRequirement()



from checkov.common.models.enums import CheckResult

from checkov.gitlab_ci.checks.base_gitlab_ci_check import BaseGitlabCICheck
from checkov.yaml_doc.enums import BlockType


class SuspectCurlInScript(BaseGitlabCICheck):
    def __init__(self):
        name = "Suspicious use of curl with CI environment variables in script"
        id = "CKV_GITLABCI_1"
        super().__init__(
            name=name,
            id=id,
            block_type=BlockType.ARRAY,
            supported_entities=['*.script[]']
        )

    def scan_entity_conf(self, conf):
        if "curl" in conf:
            badstuff = ['curl','$CI_']
            lines = conf.split("\n")
            for line in lines:
                if all(x in line for x in badstuff):
                    return CheckResult.FAILED, conf
        return CheckResult.PASSED, conf

check = SuspectCurlInScript()



from checkov.common.images.image_referencer import ImageReferencer, Image
from checkov.common.output.report import CheckType
from checkov.gitlab_ci.checks.registry import registry
from checkov.yaml_doc.runner import Runner as YamlRunner

class Runner(YamlRunner, ImageReferencer):
    check_type = CheckType.GITLAB_CI

    def require_external_checks(self):
        return False

    def import_registry(self):
        return registry

    def _parse_file(self, f):
        if self.is_workflow_file(f):
            return super()._parse_file(f)

    def is_workflow_file(self, file_path):
        """
        :return: True if the file mentioned is in the gitlab workflow name .gitlab-ci.yml. Otherwise: False
        """
        return file_path.endswith((".gitlab-ci.yml",".gitlab-ci.yaml"))

    def included_paths(self):
        return [".gitlab-ci.yml",".gitlab-ci.yaml"]

    def get_images(self, file_path):
        """
        Get container images mentioned in a file
        :param file_path: File to be inspected
        GitLab a workflow file can have a job and services run within a container.

        in the following sample file we can see a node:14.16 image:

        default:
            image:
                name: ruby:2.6
                entrypoint: ["/bin/bash"]

            image: nginx:1.18

            services:
                - name: privateregistry/stuff/my-postgres:11.7
                  alias: db-postgres
                - name: redis:latest  
                - nginx:1.17
        Source: https://docs.gitlab.com/ee/ci/docker/using_docker_images.html

        :return: List of container image short ids mentioned in the file.
        Example return value for a file with node:14.16 image: ['sha256:6a353e22ce']
        """

        images = set()
        imagesKeys = ["image","services"]
        workflow, workflow_line_numbers = self._parse_file(file_path)

        for job_name, job_object in workflow.items():
            if isinstance(job_object, dict):
                start_line = job_object.get('__startline__', 0)
                end_line = job_object.get('__endline__', 0)
                for key, subjob in job_object.items():
                    if key in imagesKeys:
                        if isinstance(subjob, dict):
                            start_line = subjob.get('__startline__', 0)
                            end_line = subjob.get('__endline__', 0)
                            imagename = subjob['name']
                        elif isinstance(subjob, str):
                            imagename = subjob
                        elif isinstance(subjob, list):
                            for service in subjob:
                                if isinstance(service, dict):
                                    start_line = service.get('__startline__', 0)
                                    end_line = service.get('__endline__', 0)
                                    imagename = service['name']
                                elif isinstance(service, str):
                                    imagename = service
                                if imagename:
                                    image_id = self.inspect(imagename)
                                    if image_id:
                                        image_obj = Image(file_path=file_path, name=imagename, image_id=image_id, start_line=start_line,
                                                        end_line=end_line)
                                        images.add(image_obj)
                                    imagename = ""      
                        if imagename:
                            image_id = self.inspect(imagename)
                            if image_id:
                                image_obj = Image(file_path=file_path, name=imagename, image_id=image_id, start_line=start_line,
                                                  end_line=end_line)
                                images.add(image_obj)
                            imagename = ""
        return images



from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class PolicyNoDSRI(BaseResourceCheck):
    def __init__(self):
        name = "Ensure DSRI is not enabled within security policies"
        id = "CKV_PAN_4"
        supported_resources = ['panos_security_policy','panos_security_rule_group']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
    
        # Check there is a rule defined in the resource
        if 'rule' in conf:

            # Report the area of evaluation
            self.evaluated_keys = ['rule']

            # Get all the rules defined in the resource
            rules = conf.get('rule')

            # Iterate over each rule
            for secrule in rules:

                # Check if DSRI is defined in the resource
                if 'disable_server_response_inspection' in secrule:

                    # If DSRI is defined, get the value
                    dsriflag = secrule.get('disable_server_response_inspection')

                    # Setting DSRI to true is a fail as server-to-client inspection will be disabled
                    if dsriflag[0]:
                        return CheckResult.FAILED

            # The other value for DSRI is false, which is a pass
            # Also, if the DSRI attribute is not explicitly set, the default is false, which is also a pass
            return CheckResult.PASSED

        # If there's no rules we have nothing to check
        return CheckResult.UNKNOWN

check = PolicyNoDSRI()



import os
from pathlib import Path
from typing import Dict, Any

from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
from checkov.common.bridgecrew.vulnerability_scanning.integrations.twistcli import TwistcliIntegration
from checkov.common.util.str_utils import removeprefix


class DockerImageScanningIntegration(TwistcliIntegration):
    async def report_results_async(
        self,
        twistcli_scan_result: Dict[str, Any],
        bc_platform_integration: BcPlatformIntegration,
        bc_api_key: str,
        file_path: Path,
        **kwargs: Any,
    ) -> int:
        return await super().report_results_async(
            twistcli_scan_result=twistcli_scan_result,
            bc_platform_integration=bc_platform_integration,
            bc_api_key=bc_api_key,
            file_path=file_path,
            file_content=kwargs["file_content"],
            docker_image_name=kwargs["docker_image_name"],
        )

    # can be removed, if image scanning is also using report_results_asyncio
    # make sure to remove it only after validating it isn't used from the platform
    def report_results(
        self,
        twistcli_scan_result: Dict[str, Any],
        file_path: Path,
        **kwargs: Any,
    ) -> None:
        super().report_results(
            twistcli_scan_result=twistcli_scan_result,
            file_path=file_path,
            file_content=kwargs['file_content'],
            docker_image_name=kwargs['docker_image_name'],
        )

    def create_report(
        self,
        twistcli_scan_result: Dict[str, Any],
        bc_platform_integration: BcPlatformIntegration,
        file_path: Path,
        file_content: str,
        docker_image_name: str,
    ) -> Dict[str, Any]:
        results_dict = self._get_results_dict(twistcli_scan_result)
        payload = {
            "dockerImageName": docker_image_name,
            "dockerFilePath": removeprefix(str(file_path), os.getenv("BC_ROOT_DIR", "")),
            "dockerFileContent": file_content,
            "type": "Image",
            "sourceId": bc_platform_integration.repo_id,
            "branch": bc_platform_integration.repo_branch,
            "sourceType": bc_platform_integration.bc_source.name,
            "vulnerabilities": self.get_vulnerabilities_for_report(results_dict),
            "packages": self.get_packages_for_report(results_dict),
        }
        if bc_platform_integration.cicd_details:
            payload["cicdDetails"] = bc_platform_integration.cicd_details
        return payload

    @staticmethod
    def _get_results_dict(twistcli_scan_result: Dict[str, Any]) -> Dict[str, Any]:
        try:
            results = twistcli_scan_result["results"]
            # we expect that there is exactly 1 scanned dokcer-image.
            if len(results) != 1:
                raise Exception(f'expected length of \"results\": 1. found: {len(results)}', results)
        except KeyError:
            raise Exception("the key \"results\" expected to be existing in:", twistcli_scan_result)
        return results[0]


docker_image_scanning_integration = DockerImageScanningIntegration()



from typing import Any, List, Dict

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class GoogleSubnetworkLoggingEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Private google access is enabled for IPV6"
        id = "CKV_GCP_76"
        supported_resources = ("google_compute_subnetwork",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:

        stack = conf.get("stack_type")
        if stack and stack[0] != "IPV4_IPV6":
            return CheckResult.UNKNOWN

        return super().scan_resource_conf(conf)

    def get_inspected_key(self) -> str:
        return "private_ipv6_google_access"


check = GoogleSubnetworkLoggingEnabled()



import logging
from typing import Dict

from checkov.common.parsers.json import parse as json_parse
from checkov.common.parsers.node import DictNode

LOGGER = logging.getLogger(__name__)


def parse(filename, out_parsing_errors: Dict[str, str]):
    """
        Decode filename into an object
    """
    (template, template_lines) = json_parse(filename, out_parsing_errors=out_parsing_errors)

    if (
        template is not None
        and isinstance(template, DictNode)
        and 'terraform_version' in template
        and 'planned_values' in template
    ):
        return template, template_lines
    return None, None



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class CPULimits(BaseResourceCheck):
    def __init__(self) -> None:
        name = "CPU Limits should be set"
        id = "CKV_K8S_11"
        supported_resources = ["kubernetes_pod"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if "spec" not in conf:
            self.evaluated_keys = [""]
            return CheckResult.FAILED
        spec = conf['spec'][0]

        containers = spec.get("container")
        for idx, container in enumerate(containers):
            if type(container) != dict:
                return CheckResult.UNKNOWN
            if container.get("resources"):
                resources = container.get("resources")[0]
                if resources.get('limits'):
                    limits = resources.get('limits')[0]
                    if limits.get('cpu'):
                        return CheckResult.PASSED
                    self.evaluated_keys = [f'spec/[0]/container/[{idx}]/resources/[0]/limits']
                    return CheckResult.FAILED
                self.evaluated_keys = [f'spec/[0]/container/[{idx}]/resources']
                return CheckResult.FAILED
            self.evaluated_keys = [f'spec/[0]/container/[{idx}]']
            return CheckResult.FAILED
        return CheckResult.PASSED


check = CPULimits()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_list
from checkov.cloudformation.parser.cfn_keywords import ConditionFunctions, IntrinsicFunctions

supported_policy_prefixes = {
    # ALBs support TLS v1.2
    'HTTPS': ("ELBSecurityPolicy-FS-1-2", "ELBSecurityPolicy-TLS-1-2"),
    # NLBs support TLS v1.2 and 1.3
    'TLS': ("ELBSecurityPolicy-TLS13-1-3-2021-06", "ELBSecurityPolicy-TLS13-1-2", "ELBSecurityPolicy-FS-1-2", "ELBSecurityPolicy-TLS-1-2")
}

class ALBListenerTLS12(BaseResourceCheck):

    def __init__(self):
        name = "Ensure that Load Balancer Listener is using at least TLS v1.2"
        id = "CKV_AWS_103"
        supported_resources = ['AWS::ElasticLoadBalancingV2::Listener']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            validates that ElasticLoadBalancing V2 Listener is using at least TLS v1.2
            https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listener.html
        :param conf: aws_alb_listener configuration
        :return: <CheckResult>
        """

        if 'Properties' in conf.keys():
            if 'Protocol' in conf['Properties'].keys():
                # Check SslPolicy only if protocol is HTTPS (ALB) or TLS (NLB).
                # Other protocols are not interesting within the context of this check.
                protocol = conf['Properties']['Protocol']
                if protocol in ('HTTPS', 'TLS'):
                    if 'SslPolicy' in conf['Properties'].keys():
                        if conf['Properties']['SslPolicy'].startswith(supported_policy_prefixes[protocol]):
                            return CheckResult.PASSED
                    return CheckResult.FAILED
                elif conf['Properties']['Protocol'] in ('TCP', 'UDP', 'TCP_UDP'):
                    return CheckResult.PASSED
                for idx_action, action in enumerate(conf['Properties']['DefaultActions']):
                    if action in ConditionFunctions.__dict__.values() or action in IntrinsicFunctions.__dict__.values():
                        return CheckResult.UNKNOWN
                    redirects = action.get("RedirectConfig", [])
                    for idx_redirect, redirect in enumerate(force_list(redirects)):
                        if redirect.get("Protocol", []) == 'HTTPS':
                            return CheckResult.PASSED
        return CheckResult.FAILED

check = ALBListenerTLS12()



from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class PolicyNoSrcAnyDstAny(BaseResourceCheck):
    def __init__(self):
        name = "Ensure security rules do not have 'source_addresses' and 'destination_addresses' both containing values of 'any' "
        id = "CKV_PAN_7"
        supported_resources = ['panos_security_policy','panos_security_rule_group']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
    
        # Check there is a rule defined in the resource
        if 'rule' in conf:

            # Report the area of evaluation
            self.evaluated_keys = ['rule']

            # Get all the rules defined in the resource
            rules = conf.get('rule')

            # Iterate over each rule
            for secrule in rules:

                # Check if source_addresses is defined in the resource
                if 'source_addresses' in secrule:

                    # If source_addresses is defined, get the value
                    source_addresses = secrule.get('source_addresses')
                    
                    # source_addresses can have a list of values, so iterate over each value
                    for src_address in source_addresses[0]:
                    
                        # The value "any" is overly permissive for source_addresses only if combined with destination_address=any...
                        if src_address == "any":

                            # ...so check if destination_addresses is defined in the resource
                            if 'destination_addresses' in secrule:

                                # If destination_addresses is defined, get the value
                                destination_addresses = secrule.get('destination_addresses')

                                # destination_addresses can have a list of values, so iterate over each value
                                for dst_address in destination_addresses[0]:
                                
                                    # If the value of destination_addresses is also "any" as well as source_addresses=any, this is overly permissive
                                    if dst_address == "any":
                                        return CheckResult.FAILED
                            
                            else:
                                # If "destination_addresses" attribute is not defined, this is not valid and will fail during Terraform plan stage, and should therefore be a fail
                                return CheckResult.FAILED

                else:
                    # If "source_addresses" attribute is not defined, this is not valid and will fail during Terraform plan stage, and should therefore be a fail
                    return CheckResult.FAILED
                
            # We have been through all rules and not found one with failure where source_addresses and destination_addresses both have the value 'any', so this is a pass
            return CheckResult.PASSED
            
        # If there's no rules we have nothing to check
        return CheckResult.UNKNOWN


check = PolicyNoSrcAnyDstAny()



from checkov.arm.checks.resource.NSGRulePortAccessRestricted import NSGRulePortAccessRestricted


class NSGRuleSSHAccessRestricted(NSGRulePortAccessRestricted):
    def __init__(self):
        super().__init__(name="Ensure that SSH access is restricted from the internet", check_id="CKV_AZURE_10", port=22)


check = NSGRuleSSHAccessRestricted()



from __future__ import annotations

from typing import Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.checks.enums import BlockType
from checkov.common.parsers.node import DictNode
from checkov.openapi.checks.base_openapi_check import BaseOpenapiCheck


class SecurityDefinitions(BaseOpenapiCheck):
    def __init__(self) -> None:
        id = "CKV_OPENAPI_1"
        name = "Ensure that securityDefinitions is defined and not empty."
        categories = [CheckCategories.API_SECURITY]
        supported_resources = ['securityDefinitions']
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_resources,
                         block_type=BlockType.DOCUMENT)

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:  # type:ignore[override]  # return type is different than the base class
        self.evaluated_keys = ["securityDefinitions"]
        if "securityDefinitions" not in conf:
            return CheckResult.FAILED, conf

        security_definitions = conf["securityDefinitions"]
        if not security_definitions or (not isinstance(security_definitions, DictNode) and len(security_definitions) <= 2):
            return CheckResult.FAILED, security_definitions
        return CheckResult.PASSED, security_definitions

check = SecurityDefinitions()



from checkov.common.models.consts import ANY_VALUE
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class GoogleComputeDiskEncryption(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure VM disks for critical VMs are encrypted with Customer Supplied Encryption Keys (CSEK)"
        id = "CKV_GCP_37"
        supported_resources = ['google_compute_disk']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'disk_encryption_key'

    def get_expected_value(self):
        return ANY_VALUE


check = GoogleComputeDiskEncryption()



from __future__ import annotations

from typing import Any, cast

from checkov.common.checks.base_check_registry import BaseCheckRegistry
from checkov.common.util.banner import banner as checkov_banner
import click
import jinja2
import os

CHECKOV_ROOT_DIRECTORY = os.path.join(".", "checkov")
TEMPLATE_DIRECTORY = os.path.join(os.path.dirname(__file__), "templates")


class Prompt():
    ACTIONS = ["add"]
    CHECK_CLASS = ["terraform"]
    CATEGORIES = ["application_security", "backup_and_recovery" "convention", "encryption",
                  "general_security", "iam", "kubernetes", "logging", "networking", "secrets"]
    TERRAFORM_OBJECT_TYPE = ["data", "provider", "resource"]
    PROVIDERS = ["azure", "aws", "gcp"]

    PROMPTS = {
        "chosen_action": {
            "text": 'What action would you like to take?',
            "type": click.Choice(ACTIONS),
            "default": "add",
            "sub_prompts": [
                {
                    "title": {
                        "text": 'Enter the title of your new check (without a .py)',
                        "type": str,
                        "default": "MyNewTest"
                    },
                    "category": {
                        "text": 'Select a category for this check',
                        "type": click.Choice(CATEGORIES),
                        "default": "iam"
                    },
                    "desc": {
                        "text": 'Describe what this check does',
                        "type": str,
                        "default": "Ensure that X does Y..."
                    },
                    "check_class": {
                        "prompt_if": 'add',
                        "text": 'What kind of check would you like to add?',
                        "type": click.Choice(CHECK_CLASS),
                        "default": "terraform",
                        "sub_prompts": [
                            {  # Terraform
                                "provider": {
                                    "prompt_if": 'terraform',
                                    "text": 'Select the cloud provider this will run on',
                                    "type": click.Choice(PROVIDERS),
                                    "default": "aws",
                                    "sub_prompts": [
                                        {  # AWS
                                            "context": {
                                                "prompt_if": 'aws',
                                                "text": 'Select a terraform object for this check',
                                                "type": click.Choice(TERRAFORM_OBJECT_TYPE),
                                                "default": "resource",
                                                "sub_prompts": [
                                                    {
                                                        "supported_resource": {
                                                            "prompt_if": 'resource',
                                                            "text": 'Enter the terraform object type',
                                                            "type": str,
                                                            "default": "aws_iam_policy"
                                                        },
                                                    },
                                                    {
                                                        "supported_resource": {
                                                            "prompt_if": 'data',
                                                            "text": 'Enter the terraform object type',
                                                            "type": str,
                                                            "default": "aws_iam_policy_document"
                                                        },
                                                    },
                                                    {
                                                        "supported_resource": {
                                                            "prompt_if": 'provider',
                                                            "text": 'Enter the terraform object type',
                                                            "type": str,
                                                            "default": "aws"
                                                        },
                                                    }
                                                ]
                                            }
                                        },
                                        {  # Azure
                                            "context": {
                                                "prompt_if": 'azure',
                                                "text": 'Select a terraform object for this check',
                                                "type": click.Choice(['resource']),
                                                "default": "resource",
                                                "sub_prompts": [
                                                    {
                                                        "supported_resource": {
                                                            "prompt_if": 'resource',
                                                            "text": 'Enter the terraform object type',
                                                            "type": str,
                                                            "default": "azurerm_policy_definition"
                                                        },
                                                    }
                                                ]
                                            }
                                        },
                                        {  # GCP
                                            "context": {
                                                "prompt_if": 'gcp',
                                                "text": 'Select a terraform object for this check',
                                                "type": click.Choice(['resource']),
                                                "default": "resource",
                                                "sub_prompts": [
                                                    {
                                                        "supported_resource": {
                                                            "prompt_if": 'resource',
                                                            "text": 'Enter the terraform object type',
                                                            "type": str,
                                                            "default": "google_project_iam_policy"
                                                        },
                                                    }
                                                ]
                                            }
                                        },
                                    ]
                                }
                            }
                        ]
                    }
                }
            ]
        }
    }

    def __init__(self) -> None:
        print(checkov_banner)
        self.responses: dict[str, Any] = {}
        self.prompt()

    # Recurse over our prompt, populating new class attributes from the keys and
    # user-supplied answers
    def prompt(self, prompt_map: dict[str, Any] | None = None, prompt_if: str | None = None) -> None:
        if not prompt_map:
            prompt_map = Prompt.PROMPTS

        for k, v in prompt_map.items():
            if "prompt_if" not in v or v["prompt_if"] == prompt_if:
                # Prompt the user
                text = cast(str, v["text"])
                p = click.prompt(text, type=v["type"], default=v["default"])

                # Create the action on our object
                if k == 'chosen_action':
                    self.chosen_action = p

                # Record user responses
                self.responses[k] = p
                print()  # Newline for readability

                # Call prompt() again on any sub_prompts
                if "sub_prompts" in v and isinstance(v["sub_prompts"], list):
                    for sub_prompt in v["sub_prompts"]:
                        self.prompt(sub_prompt, prompt_if=p)

    def template_env(self) -> jinja2.Environment:
        template_loader = jinja2.FileSystemLoader(searchpath=TEMPLATE_DIRECTORY)
        return jinja2.Environment(loader=template_loader, autoescape=True)


class Check(Prompt):
    def __init__(self, user_responses: dict[str, Any] | None = None) -> None:
        if not user_responses:
            user_responses = {}

        self.chosen_action = user_responses.get("chosen_action", None)
        self.title = user_responses.get("title", None)
        self.category = user_responses.get("category", None)
        self.desc = user_responses.get("desc", None)
        self.check_class = user_responses.get("check_class", None)
        self.provider = user_responses.get("provider", None)
        self.context = user_responses.get("context", None)
        self.supported_resource = user_responses.get("supported_resource", None)

    def action(self) -> None:
        # Call the user-selected action (add, remove, modify - leaves room for more actions)
        getattr(self, self.chosen_action)()

    def add(self) -> None:
        print("Please ensure you are at the root of the Checkov repository before completing this prompt")

        self.populate_templates()
        self.create_check()
        self.create_unit_test_stubs()
        self.print_instructions()

    def populate_templates(self) -> None:
        # Fetch the tf template for unit tests
        tf_unit_test_template = self.template_env().get_template("unittest-terraform.jinja2")
        self.tf_unit_test_template = tf_unit_test_template.render(
            supported_resource=self.supported_resource, context=self.context)

        # Fetch the python template for unit tests
        python_unit_test_template = self.template_env().get_template("unittest-python.jinja2")
        self.python_unit_test_template = python_unit_test_template.render(
            provider=self.provider, title=self.title, supported_resource=self.supported_resource, check_class=self.check_class, context=self.context)

        # Fetch the init template
        init_template = self.template_env().get_template("init.jinja2")
        self.init_template = init_template.render()

        # Fetch the check template
        check_template = self.template_env().get_template(f"{self.context}.jinja2")
        new_index = self.get_latest_id_for_provider() + 1  # Find the latest ID and increment

        self.template = check_template.render(title=self.title, provider=self.provider.upper(),
                                              supported_resource=self.supported_resource, category=self.category.upper(),
                                              desc=self.desc, index=new_index)

    def get_latest_id_for_provider(self) -> int:
        max_id = 0
        try:
            for ck in BaseCheckRegistry.get_all_registered_checks():
                if ck.id.startswith(f"CKV_{self.provider.upper()}"):
                    curr_id_num = int(ck.id.split("_")[2])
                    if curr_id_num > max_id:
                        max_id = curr_id_num
        except Exception:
            print("Unable to automatically find latest Check ID, please set manually")
            max_id = 999999

        return max_id

    def create_check(self) -> None:
        # Create check in the checks directory
        ck_loc = os.path.abspath(os.path.join(CHECKOV_ROOT_DIRECTORY,
                                              self.check_class, "checks", self.context, self.provider.lower()))
        print(f"Creating Check {self.title}.py in {ck_loc}")

        # Set path, make directory path if necessary
        full_path = os.path.join(ck_loc, f"{self.title}.py")
        os.makedirs(ck_loc, exist_ok=True)

        # Write file
        with open(full_path, "w") as f:
            f.write(self.template)

        print(f"\tSuccessfully created {full_path}")

    def create_unit_test_stubs(self) -> None:
        base = os.path.abspath(os.path.join(CHECKOV_ROOT_DIRECTORY, os.path.pardir,
                                            "tests", self.check_class, "checks", self.context, self.provider.lower()))
        print(f"Creating Unit Test Stubs for {self.title} in {base}")

        # Create Terraform stub from Template
        new_dir = os.path.join(base, f"example_{self.title}")
        os.makedirs(new_dir, exist_ok=True)

        tf_loc = os.path.join(new_dir, f"{self.title}.tf")
        with open(tf_loc, "w") as f:
            f.write(self.tf_unit_test_template)

        print(f"\tSuccessfully created {tf_loc}")

        py_loc = os.path.join(base, f"test_{self.title}.py")

        with open(py_loc, "w") as f:
            f.write(self.python_unit_test_template)

        print(f"\tSuccessfully created {py_loc}")

    def print_instructions(self) -> None:
        print("\nNext steps:")
        print("\t1) Edit your new check located in the checks/ directory listed above")
        print("\t2) Add both a PASS and FAIL unit test to the newly created unit test under the tests/ directory to show others how to fix failures")



import logging
import re
from abc import ABC, abstractmethod
from collections import defaultdict
from itertools import islice
from pathlib import Path
from typing import List, Dict, Any, Tuple, Optional

import dpath.util

from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import integration as metadata_integration
from checkov.common.comment.enum import COMMENT_REGEX
from checkov.common.models.enums import ContextCategories
from checkov.terraform.context_parsers.registry import parser_registry

OPEN_CURLY = "{"
CLOSE_CURLY = "}"


class BaseContextParser(ABC):
    definition_type = ""
    tf_file = ""
    tf_file_path: Optional[Path] = None
    file_lines: List[Tuple[int, str]] = []
    filtered_lines: List[Tuple[int, str]] = []
    filtered_line_numbers: List[int] = []
    context: Dict[str, Any] = defaultdict(dict)

    def __init__(self, definition_type: str) -> None:
        # bc_integration.setup_http_manager()
        self.logger = logging.getLogger("{}".format(self.__module__))
        if definition_type.upper() not in ContextCategories.__members__:
            self.logger.error("Terraform context parser type not supported yet")
            raise Exception()
        self.definition_type = definition_type
        parser_registry.register(self)

    @abstractmethod
    def get_entity_context_path(self, entity_block: Dict[str, Dict[str, Any]]) -> List[str]:
        """
        returns the entity's path in the context parser
        :param entity_block: entity definition block
        :return: list of nested entity's keys in the context parser
        """
        raise NotImplementedError

    def _is_block_signature(self, line_num: int, line_tokens: List[str], entity_context_path: List[str]) -> bool:
        """
        Determine if the given tokenized line token is the entity signature line
        :param line_num: The line number in the file
        :param line_tokens: list of line tokens
        :param entity_context_path: the entity's path in the context parser
        :return: True/False
        """
        block_type = self.get_block_type()
        return all(x in line_tokens for x in [block_type] + entity_context_path)

    @staticmethod
    def _trim_whitespaces_linebreaks(text: str) -> str:
        return text.strip()

    def _filter_file_lines(self) -> List[Tuple[int, str]]:
        parsed_file_lines = [(ind, self._trim_whitespaces_linebreaks(line)) for (ind, line) in self.file_lines]
        self.filtered_lines = [(ind, line) for (ind, line) in parsed_file_lines if line]
        self.filtered_line_numbers = [ind for ind, _ in self.filtered_lines]
        return self.filtered_lines

    def _read_file_lines(self) -> List[Tuple[int, str]]:
        with open(self.tf_file, "r") as file:
            file.seek(0)
            file_lines = [(ind + 1, line) for ind, line in enumerate(file.readlines())]
            return file_lines

    @staticmethod
    def is_optional_comment_line(line: str) -> bool:
        line_without_whitespace = line.replace(" ", "")
        return "checkov:skip=" in line_without_whitespace or "bridgecrew:skip=" in line_without_whitespace

    def _collect_skip_comments(self, definition_blocks: List[Dict[str, Any]]) -> Dict[str, Any]:
        """
        Collects checkov skip comments to all definition blocks
        :param definition_blocks: parsed definition blocks
        :return: context enriched with with skipped checks per skipped entity
        """
        bc_id_mapping = metadata_integration.bc_to_ckv_id_mapping
        comments = [
            (
                line_num,
                {
                    "id": match.group(2),
                    "suppress_comment": match.group(3)[1:] if match.group(3) else "No comment provided",
                },
            )
            for (line_num, x) in self.file_lines
            if self.is_optional_comment_line(x)
            for match in [re.search(COMMENT_REGEX, x)]
            if match
        ]
        for entity_block in definition_blocks:
            skipped_checks = []
            entity_context_path = self.get_entity_context_path(entity_block)
            entity_context = self.context
            found = True
            for k in entity_context_path:
                if k in entity_context:
                    entity_context = entity_context[k]
                else:
                    logging.warning(f'Failed to find context for {".".join(entity_context_path)}')
                    found = False
                    break
            if not found:
                continue
            for (skip_check_line_num, skip_check) in comments:
                if "start_line" in entity_context and "end_line" in entity_context \
                        and entity_context["start_line"] < skip_check_line_num < entity_context["end_line"]:
                    # No matter which ID was used to skip, save the pair of IDs in the appropriate fields
                    if bc_id_mapping and skip_check["id"] in bc_id_mapping:
                        skip_check["bc_id"] = skip_check["id"]
                        skip_check["id"] = bc_id_mapping[skip_check["id"]]
                    elif metadata_integration.check_metadata:
                        skip_check["bc_id"] = metadata_integration.get_bc_id(skip_check["id"])
                    skipped_checks.append(skip_check)
            dpath.new(self.context, entity_context_path + ["skipped_checks"], skipped_checks)
        return self.context

    def _compute_definition_end_line(self, start_line_num: int) -> int:
        """Given the code block's start line, compute the block's end line
        :param start_line_num: code block's first line number (the signature line)
        :return: the code block's last line number
        """
        parsed_file_lines = self.filtered_lines
        start_line_idx = self.filtered_line_numbers.index(start_line_num)
        i = 0
        end_line_num = 0
        for (line_num, line) in islice(parsed_file_lines, start_line_idx, None):
            if OPEN_CURLY in line:
                i += line.count(OPEN_CURLY)
            if CLOSE_CURLY in line:
                i -= line.count(CLOSE_CURLY)
                if i == 0:
                    end_line_num = line_num
                    break
        return end_line_num

    def run(
            self, tf_file: str, definition_blocks: List[Dict[str, Any]], collect_skip_comments: bool = True
    ) -> Dict[str, Any]:
        # TF files for loaded modules have this formation:  <file>[<referrer>#<index>]
        # Chop off everything after the file name for our purposes here
        if tf_file.endswith("]") and "[" in tf_file:
            self.tf_file = tf_file[: tf_file.index("[")]
            self.tf_file_path = Path(tf_file[: tf_file.index("[")])
        else:
            self.tf_file = tf_file
            self.tf_file_path = Path(tf_file)
        self.context = defaultdict(dict)
        self.file_lines = self._read_file_lines()
        self.context = self.enrich_definition_block(definition_blocks)
        if collect_skip_comments:
            self.context = self._collect_skip_comments(definition_blocks)
        return self.context

    def get_block_type(self) -> str:
        return self.definition_type

    @staticmethod
    def _clean_line(line: str) -> str:
        res = line.replace('"', " ")
        if '"{' in res:
            res = res.split("{")[0]
        return res

    @abstractmethod
    def enrich_definition_block(self, definition_blocks: List[Dict[str, Any]]) -> Dict[str, Any]:
        """
        Enrich the context of a Terraform block
        :param definition_blocks: Terraform block, key-value dictionary
        :return: Enriched block context
        """
        pass



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ECSClusterLoggingEncryptedWithCMK(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Cluster logging with CMK"
        id = "CKV_AWS_224"
        supported_resources = ['aws_ecs_cluster']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if conf.get("configuration"):
            if conf.get("configuration")[0].get('execute_command_configuration'):
                command_conf = conf.get("configuration")[0].get('execute_command_configuration')[0]
                if not command_conf.get('logging') == ['NONE']:
                    if command_conf.get('kms_key_id'):
                        if command_conf.get('log_configuration'):
                            log_conf = command_conf.get('log_configuration')[0]
                            if log_conf.get('cloud_watch_encryption_enabled') == [True] or \
                                    log_conf.get('s3_bucket_encryption_enabled') == [True]:
                                return CheckResult.PASSED
                        return CheckResult.FAILED
                    else:
                        return CheckResult.FAILED

        return CheckResult.UNKNOWN


check = ECSClusterLoggingEncryptedWithCMK()
